#include <stdint.h>
#include <inttypes.h>
#include <time.h>
#include <unistd.h>
#include <string>
#include <sstream>
#include <fstream>
#include "all_in_one.hpp"
#include "mtgp32.hpp"
#include "parse_opt.h"
#include "mtgp32ex.h"

using namespace std;
using namespace MTToolBox;
using namespace mtgp;

typedef linear_generator<uint32_t, mtgp32> lg32;
typedef search_temper<lg32, uint32_t, 32, 4, 23, 5> st32;

bool indexed_search(mtgp_options& opt, bool first);

int main(int argc, char** argv) {
    mtgp_options opt;
    bool first = true;
    bool parse = parse_opt(opt, argc, argv);
    if (!parse) {
	return -1;
    }
    while (opt.count > 0) {
	if (indexed_search(opt, first)) {
	    opt.id += 1;
	    opt.count -= 1;
	} else {
	    return -1;
	}
	first = false;
    }
    return 0;
}

bool indexed_search(mtgp_options& opt, bool first) {
    mtgp32 mtgp(opt.mexp, opt.id);
    shared_ptr<lg32> g(new lg32(mtgp));

    if (opt.verbose) {
	time_t t = time(NULL);
	cout << "search start at " << ctime(&t);
	cout << "mexp:" << dec << opt.mexp << endl;
	cout << "id:" << dec << opt.id << endl;
    }
    all_in_one<uint32_t, lg32, st32> all;
    st32 st;
    if (all.search(*g, st, opt.seed, opt.verbose)) {
	g = all.get_rand();
	shared_ptr<mtgp32> rand = g->get_random();
	mtgp_param<uint32_t> param = rand->get_param();
	string sha1 = all.get_sha1();
	mtgp32ex ex32(param, sha1, 1);
	for (int i = 0; i < 10; i++) {
	    cout << ex32.next() << " ";
	}
	cout << endl;
	return true;
    } else {
	cout << "search failed" << endl;
	return false;
    }
}
